@@ -1,6 +1,6 @@
 # -*- coding:utf-8 -*-
 
-
+import re
 import os
 import sys
 import shutil
@@ -517,8 +517,8 @@ class BflbIotTool(object):
             efusemaskfile = img_cfg.get("Img_Cfg", "efuse_mask_file")
             cfg = BFConfigParser()
             cfg.read(self.eflash_loader_cfg)
-            cfg.set('EFUSE_CFG', 'file', efusefile.replace("\\", "/"))
-            cfg.set('EFUSE_CFG', 'maskfile', efusemaskfile.replace("\\", "/"))
+            cfg.set('EFUSE_CFG', 'file', os.path.relpath(efusefile, app_path).replace("\\", "/"))
+            cfg.set('EFUSE_CFG', 'maskfile', os.path.relpath(efusemaskfile, app_path).replace("\\", "/"))
             cfg.write(self.eflash_loader_cfg, 'w')
             self.efuse_load_en = True
         else:
@@ -617,31 +617,31 @@ class BflbIotTool(object):
                 whole_img_len = 0x2000 + os.path.getsize(
                     chipnamedir + "/img_create_iot/img_boot2.bin")
                 whole_img_data = bl_create_flash_default_data(whole_img_len)
-                filedata = bl_get_file_data([chipnamedir + "/img_create_iot/bootinfo_boot2.bin"])[0]
+                filedata = bl_get_file_data([chipname + "/img_create_iot/bootinfo_boot2.bin"])[0]
                 whole_img_data[0:len(filedata)] = filedata
-                filedata = bl_get_file_data([chipnamedir + "/img_create_iot/img_boot2.bin"])[0]
+                filedata = bl_get_file_data([chipname + "/img_create_iot/img_boot2.bin"])[0]
                 whole_img_data[0x2000:0x2000 + len(filedata)] = filedata
                 fp = open(whole_img, 'wb+')
                 fp.write(whole_img_data)
                 fp.close()
-                d_files.append(chipnamedir + "/img_create_iot/whole_img_boot2.bin")
+                d_files.append(chipname + "/img_create_iot/whole_img_boot2.bin")
                 d_addrs.append("00000000")
             else:
-                d_files.append(chipnamedir + "/img_create_iot/bootinfo_boot2.bin")
+                d_files.append(chipname + "/img_create_iot/bootinfo_boot2.bin")
                 d_addrs.append("00000000")
-                d_files.append(chipnamedir + "/img_create_iot/img_boot2.bin")
+                d_files.append(chipname + "/img_create_iot/img_boot2.bin")
                 d_addrs.append("00002000")
         elif chiptype == "bl702":
             bflb_utils.printf("========= copy bootinfo_boot2.bin =========")
-            bflb_utils.copyfile(chipnamedir + "/img_create_iot/bootinfo.bin",
-                                chipnamedir + "/img_create_iot/bootinfo_boot2.bin")
-            d_files.append(chipnamedir + "/img_create_iot/bootinfo_boot2.bin")
+            bflb_utils.copyfile(chipname + "/img_create_iot/bootinfo.bin",
+                                chipname + "/img_create_iot/bootinfo_boot2.bin")
+            d_files.append(chipname + "/img_create_iot/bootinfo_boot2.bin")
             d_addrs.append("00000000")
     
         if pt_parcel is not None and len(pt_parcel) > 0 and pt_parcel['pt_new'] is True:
-            d_files.append(chipnamedir + "/partition/partition.bin")
+            d_files.append(chipname + "/partition/partition.bin")
             d_addrs.append(hex(pt_parcel['pt_addr0'])[2:])
-            d_files.append(chipnamedir + "/partition/partition.bin")
+            d_files.append(chipname + "/partition/partition.bin")
             d_addrs.append(hex(pt_parcel['pt_addr1'])[2:])
     
         if bin_file is True and 'fw_cpu0_addr' in pt_parcel:
@@ -650,19 +650,19 @@ class BflbIotTool(object):
                 whole_img_len = 0x1000 + os.path.getsize(
                     chipnamedir + "/img_create_iot/img_cpu0.bin")
                 whole_img_data = bl_create_flash_default_data(whole_img_len)
-                filedata = bl_get_file_data([chipnamedir + "/img_create_iot/bootinfo_cpu0.bin"])[0]
+                filedata = bl_get_file_data([chipname + "/img_create_iot/bootinfo_cpu0.bin"])[0]
                 whole_img_data[0:len(filedata)] = filedata
-                filedata = bl_get_file_data([chipnamedir + "/img_create_iot/img_cpu0.bin"])[0]
+                filedata = bl_get_file_data([chipname + "/img_create_iot/img_cpu0.bin"])[0]
                 whole_img_data[0x1000:0x1000 + len(filedata)] = filedata
                 fp = open(whole_img, 'wb+')
                 fp.write(whole_img_data)
                 fp.close()
-                d_files.append(chipnamedir + "/img_create_iot/whole_img_cpu0.bin")
+                d_files.append(chipname + "/img_create_iot/whole_img_cpu0.bin")
                 d_addrs.append(hex(pt_parcel['fw_cpu0_addr'])[2:])
             else:
-                d_files.append(chipnamedir + "/img_create_iot/bootinfo_cpu0.bin")
+                d_files.append(chipname + "/img_create_iot/bootinfo_cpu0.bin")
                 d_addrs.append(hex(pt_parcel['fw_cpu0_addr'])[2:])
-                d_files.append(chipnamedir + "/img_create_iot/img_cpu0.bin")
+                d_files.append(chipname + "/img_create_iot/img_cpu0.bin")
                 d_addrs.append(hex(pt_parcel['fw_cpu0_addr'] + 0x1000)[2:])
     
         if bin_file is True and 'fw_addr' in pt_parcel:
@@ -671,29 +671,29 @@ class BflbIotTool(object):
                 whole_img_len = 0x1000 + os.path.getsize(
                     chipnamedir + "/img_create_iot/img.bin")
                 whole_img_data = bl_create_flash_default_data(whole_img_len)
-                filedata = bl_get_file_data([chipnamedir + "/img_create_iot/bootinfo.bin"])[0]
+                filedata = bl_get_file_data([chipname + "/img_create_iot/bootinfo.bin"])[0]
                 whole_img_data[0:len(filedata)] = filedata
-                filedata = bl_get_file_data([chipnamedir + "/img_create_iot/img.bin"])[0]
+                filedata = bl_get_file_data([chipname + "/img_create_iot/img.bin"])[0]
                 whole_img_data[0x1000:0x1000 + len(filedata)] = filedata
                 fp = open(whole_img, 'wb+')
                 fp.write(whole_img_data)
                 fp.close()
-                d_files.append(chipnamedir + "/img_create_iot/whole_img.bin")
+                d_files.append(chipname + "/img_create_iot/whole_img.bin")
                 d_addrs.append(hex(pt_parcel['fw_addr'])[2:])
             else:
-                d_files.append(chipnamedir + "/img_create_iot/bootinfo.bin")
+                d_files.append(chipname + "/img_create_iot/bootinfo.bin")
                 d_addrs.append(hex(pt_parcel['fw_addr'])[2:])
-                d_files.append(chipnamedir + "/img_create_iot/img.bin")
+                d_files.append(chipname + "/img_create_iot/img.bin")
                 d_addrs.append(hex(pt_parcel['fw_addr'] + 0x1000)[2:])
     
         if ro_params is not None and len(ro_params) > 0 and pt_parcel['conf_addr'] is not None:
             bl_ro_device_tree.bl_ro_params_device_tree(ro_params,
-                                                       chipnamedir + "/device_tree/ro_params.dtb")
-            d_files.append(chipnamedir + "/device_tree/ro_params.dtb")
+                                                       chipname + "/device_tree/ro_params.dtb")
+            d_files.append(chipname + "/device_tree/ro_params.dtb")
             d_addrs.append(hex(pt_parcel['conf_addr'])[2:])
     
         if media is True and pt_parcel['media_addr'] is not None:
-            d_files.append(chipnamedir + "/img_create_iot/media.bin")
+            d_files.append(chipname + "/img_create_iot/media.bin")
             d_addrs.append(hex(pt_parcel['media_addr'])[2:])
     
         if mfg is True:
@@ -702,19 +702,19 @@ class BflbIotTool(object):
                 whole_img_len = 0x1000 + os.path.getsize(
                     chipnamedir + "/img_create_iot/img_mfg.bin")
                 whole_img_data = bl_create_flash_default_data(whole_img_len)
-                filedata = bl_get_file_data([chipnamedir + "/img_create_iot/bootinfo_mfg.bin"])[0]
+                filedata = bl_get_file_data([chipname + "/img_create_iot/bootinfo_mfg.bin"])[0]
                 whole_img_data[0:len(filedata)] = filedata
-                filedata = bl_get_file_data([chipnamedir + "/img_create_iot/img_mfg.bin"])[0]
+                filedata = bl_get_file_data([chipname + "/img_create_iot/img_mfg.bin"])[0]
                 whole_img_data[0x1000:0x1000 + len(filedata)] = filedata
                 fp = open(whole_img, 'wb+')
                 fp.write(whole_img_data)
                 fp.close()
-                d_files.append(chipnamedir + "/img_create_iot/whole_img_mfg.bin")
+                d_files.append(chipname + "/img_create_iot/whole_img_mfg.bin")
                 d_addrs.append(hex(pt_parcel['mfg_addr'])[2:])
             else:
-                d_files.append(chipnamedir + "/img_create_iot/bootinfo_mfg.bin")
+                d_files.append(chipname + "/img_create_iot/bootinfo_mfg.bin")
                 d_addrs.append(hex(pt_parcel['mfg_addr'])[2:])
-                d_files.append(chipnamedir + "/img_create_iot/img_mfg.bin")
+                d_files.append(chipname + "/img_create_iot/img_mfg.bin")
                 d_addrs.append(hex(pt_parcel['mfg_addr'] + 0x1000)[2:])
     
         if len(d_files) > 0 and len(d_addrs) > 0:
@@ -1169,14 +1169,15 @@ def iot_download_cmd(args):
     
     
 def run():
-    WINDOWS = sys.platform.startswith("win")
     port = None
+    ports = []
     for item in get_serial_ports():
-        port = item["port"]
-        if WINDOWS and port.startswith("COM") and len(port) > 4:
-            port = "\\\\.\\%s" % port 
-        else:
-            port = port
+        ports.append(item["port"])
+    if ports:
+        try:
+            port = sorted(ports, key=lambda x: int(re.match('COM(\d+)', x).group(1)))[0]
+        except Exception:
+            port = sorted(ports)[0]
     firmware_default = os.path.join(app_path, "img", "project.bin")
     parser = argparse.ArgumentParser(description='bflb iot tool')
     parser.add_argument('--chipname', required=True, help='chip name')
@@ -1197,7 +1198,7 @@ def run():
     if not args.port:
         bflb_utils.printf("Serial port is not found")
     else:
-        bflb_utils.printf("Serial port is " + port) 
+        bflb_utils.printf("Serial port is " + str(port)) 
     bflb_utils.printf("Baudrate is " + str(args.baudrate)) 
     bflb_utils.printf("Firmware is " + args.firmware)      
     bflb_utils.printf("==================================================")
